package org.processmining.causalactivitymatrix.dialogs;

import info.clearthought.layout.TableLayout;
import info.clearthought.layout.TableLayoutConstants;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JCheckBox;
import javax.swing.JPanel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.deckfour.xes.model.XLog;
import org.processmining.causalactivitymatrix.filters.CausalityBasedFilterManager;
import org.processmining.causalactivitymatrix.parameters.FilterLogUsingMatrixParameters;
import org.processmining.log.dialogs.ClassifierPanel;

import com.fluxicon.slickerbox.components.NiceSlider;
import com.fluxicon.slickerbox.components.NiceSlider.Orientation;
import com.fluxicon.slickerbox.factory.SlickerFactory;

public class FilterLogUsingMatrixDialog extends JPanel {

	/**
	 * 
	 */
	private static final long serialVersionUID = 3409810249042925635L;
	private final NiceSlider absoluteSlider;
	private final NiceSlider relativeSlider;
	
	public FilterLogUsingMatrixDialog(XLog log, final FilterLogUsingMatrixParameters parameters) {
		double size[][] = { { TableLayoutConstants.FILL }, { TableLayoutConstants.FILL, TableLayoutConstants.FILL, 30, 30 } };
		setLayout(new TableLayout(size));
		add(new ClassifierPanel(log.getClassifiers(), parameters), "0, 0");
		add(new FilterPanel(CausalityBasedFilterManager.getInstance().getFilters(), parameters), "0, 1");

		final JCheckBox checkBox = SlickerFactory.instance().createCheckBox("Absolute", false);
		checkBox.setSelected(parameters.isAbsolute());
		checkBox.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				if (parameters.isAbsolute()) {
					remove(absoluteSlider);
				} else {
					remove(relativeSlider);
				}
				parameters.setAbsolute(checkBox.isSelected());
				if (parameters.isAbsolute()) {
					add(absoluteSlider, "0, 3");
				} else {
					add(relativeSlider, "0, 3");
				}
				revalidate();
				repaint();
			}

		});
		checkBox.setOpaque(false);
		add(checkBox, "0, 2");

		absoluteSlider = SlickerFactory.instance().createNiceDoubleSlider("Absolute Threshold", -1.0, 1.0,
				parameters.getAbsoluteThreshold(), Orientation.HORIZONTAL);
		absoluteSlider.addChangeListener(new ChangeListener() {

			public void stateChanged(ChangeEvent e) {
				parameters.setAbsoluteThreshold(-1.0 + absoluteSlider.getSlider().getValue() * 2.0 / 10000.0);
			}
		});
		
		relativeSlider = SlickerFactory.instance().createNiceIntegerSlider("Relative Threshold", 0, 100,
				parameters.getRelativeThreshold(), Orientation.HORIZONTAL);
		relativeSlider.addChangeListener(new ChangeListener() {

			public void stateChanged(ChangeEvent e) {
				parameters.setRelativeThreshold(relativeSlider.getSlider().getValue());
			}
		});

		if (parameters.isAbsolute()) {
			add(absoluteSlider, "0, 3");
		} else {
			add(relativeSlider, "0, 3");
		}
	}
}
