package org.processmining.causalactivitymatrix.filters;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import org.processmining.causalactivitymatrix.filters.impl.CausalityBasedAverageFilter;
import org.processmining.causalactivitymatrix.filters.impl.CausalityBasedContextAverageFilter;
import org.processmining.causalactivitymatrix.filters.impl.CausalityBasedContextMinimalFilter;
import org.processmining.causalactivitymatrix.filters.impl.CausalityBasedCountFilter;
import org.processmining.causalactivitymatrix.filters.impl.CausalityBasedMinimalFilter;
import org.processmining.causalactivitymatrix.filters.impl.CausalityBasedProductFilter;
import org.processmining.causalactivitymatrix.filters.impl.CausalityBasedSingleChangeFilter;

public class CausalityBasedFilterManager {

	private static CausalityBasedFilterManager instance = null;
	private List<CausalityBasedFilter> filters;
	private CausalityBasedFilter defaultFilter;
	private boolean sorted;

	private CausalityBasedFilterManager() {
		filters = new ArrayList<CausalityBasedFilter>();
		defaultFilter = null;
		sorted = false;
		register(new CausalityBasedMinimalFilter(), false);
		register(new CausalityBasedAverageFilter(), true);
		register(new CausalityBasedContextAverageFilter(), false);
		register(new CausalityBasedContextMinimalFilter(), false);
		register(new CausalityBasedCountFilter(), false);
		register(new CausalityBasedProductFilter(), false);
		register(new CausalityBasedSingleChangeFilter(), false);
	}

	public static CausalityBasedFilterManager getInstance() {
		if (instance == null) {
			instance = new CausalityBasedFilterManager();
		}
		return instance;
	}

	public void register(CausalityBasedFilter filter, boolean isDefault) {
		filters.add(filter);
		if (isDefault) {
			defaultFilter = filter;
		}
		sorted = false;
	}

	public List<CausalityBasedFilter> getFilters() {
		if (!sorted) {
			Collections.sort(filters, new Comparator<CausalityBasedFilter>() {
				public int compare(CausalityBasedFilter f1, CausalityBasedFilter f2) {
					return f1.getName().compareTo(f2.getName());
				}
			});
			sorted = true;
		}
		return filters;
	}

	public boolean isDefault(CausalityBasedFilter filter) {
		return filter == defaultFilter;
	}

	public CausalityBasedFilter getFilter(String name) {
		if (name != null) {
			for (CausalityBasedFilter filter : filters) {
				if (name.equals(filter.getName())) {
					return filter;
				}
			}
		}
		return defaultFilter;
	}
}
