package org.processmining.causalactivitymatrix.filters;

import org.deckfour.xes.classification.XEventClassifier;
import org.deckfour.xes.model.XLog;
import org.processmining.basicutils.parameters.impl.PluginParametersImpl;
import org.processmining.log.parameters.ClassifierParameter;
import org.processmining.log.utils.XUtils;

public class CausalityBasedFilterParameters extends PluginParametersImpl implements ClassifierParameter {

	private XEventClassifier classifier;
	private double absoluteThreshold;
	private int relativeThreshold;
	private boolean isAbsolute;

	public CausalityBasedFilterParameters(XLog log) {
		super();
		setClassifier(XUtils.getDefaultClassifier(log));	
		setAbsolute(false);
		setAbsoluteThreshold(0.0);
		setRelativeThreshold(5);
	}
	
	public CausalityBasedFilterParameters(CausalityBasedFilterParameters parameters) {
		super(parameters);
		setClassifier(parameters.getClassifier());	
		setAbsolute(parameters.isAbsolute());
		setAbsoluteThreshold(parameters.getAbsoluteThreshold());
		setRelativeThreshold(parameters.getRelativeThreshold());
	}
	
	public void setClassifier(XEventClassifier classifier) {
		this.classifier = classifier;
	}

	public XEventClassifier getClassifier() {
		return classifier;
	}

	public double getAbsoluteThreshold() {
		return absoluteThreshold;
	}

	public void setAbsoluteThreshold(double threshold) {
		this.absoluteThreshold = threshold;
	}

	public boolean isAbsolute() {
		return isAbsolute;
	}

	public void setAbsolute(boolean isAbsolute) {
		this.isAbsolute = isAbsolute;
	}

	public int getRelativeThreshold() {
		return relativeThreshold;
	}

	public void setRelativeThreshold(int relativeThreshold) {
		this.relativeThreshold = relativeThreshold;
	}

	public boolean equals(Object object) {
		if (object instanceof CausalityBasedFilterParameters) {
			CausalityBasedFilterParameters parameters = (CausalityBasedFilterParameters) object;
			return super.equals(parameters) && getClassifier().equals(parameters.getClassifier()) &&
					isAbsolute() == parameters.isAbsolute() &&
					getAbsoluteThreshold() == parameters.getAbsoluteThreshold() &&
					getRelativeThreshold() == parameters.getRelativeThreshold();
		}
		return false;
	}
}
