package org.processmining.causalactivitymatrix.filters.impl;

import java.util.Collection;
import java.util.HashSet;

import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.causalactivitymatrix.filters.CausalityBasedFilter;
import org.processmining.causalactivitymatrix.filters.CausalityBasedFilterParameters;
import org.processmining.causalactivitymatrix.models.CausalActivityMatrix;

public abstract class CausalityBasedAbstractFilter implements CausalityBasedFilter {

	public XLog apply(XLog log, CausalActivityMatrix matrix, CausalityBasedFilterParameters parameters) {
		XLog filteredLog = (XLog) log.clone();
		Collection<XTrace> tracesToRemove = new HashSet<XTrace>();
		
		double threshold;
		if (parameters.isAbsolute()) {
			threshold = parameters.getAbsoluteThreshold();
		} else {
			double minValue = 1.0;
			double maxValue = -1.0;
			
			for (XTrace trace : filteredLog) {
				double value = getValue(trace, matrix, parameters);
				if (value < minValue) {
					minValue = value;
				}
				if (value > maxValue) {
					maxValue = value;
				}
			}
			threshold = minValue + (parameters.getRelativeThreshold() * (maxValue - minValue)) / 100;
		}
		for (XTrace trace : filteredLog) {
			double value = getValue(trace, matrix, parameters);
			if (value < threshold) {
				tracesToRemove.add(trace);
			}
		}
		filteredLog.removeAll(tracesToRemove);
		return filteredLog;
	}
}
