package org.processmining.causalactivitymatrix.filters.impl;

import org.deckfour.xes.classification.XEventClass;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XTrace;
import org.processmining.causalactivitymatrix.filters.CausalityBasedFilterParameters;
import org.processmining.causalactivitymatrix.models.CausalActivityMatrix;

public class CausalityBasedContextAverageFilter extends CausalityBasedAbstractFilter {

	private int contextSize;
	
	public CausalityBasedContextAverageFilter() {
		this(4);
	}
	
	public CausalityBasedContextAverageFilter(int context) {
		this.contextSize = context;
	}
	
	public double getValue(XTrace trace, CausalActivityMatrix matrix, CausalityBasedFilterParameters parameters) {
		if (trace.size() < 2) {
			return 1.0;
		}
		double sumValue = 0.0;
		XEventClass[] activities = new XEventClass[contextSize];
		for (int i = 0; i < contextSize; i++) {
			activities[i] = null;
		}
		for (XEvent event : trace) {
			for (int i = 0; i < contextSize - 1; i++) {
				activities[i] = activities[i+1];
			}
			activities[contextSize - 1] = new XEventClass(parameters.getClassifier().getClassIdentity(event), 0);
			double maxValue = -1.0;
			for (int i = 0; i < contextSize - 1; i++) {
				if (activities[i] != null) {
					double value = matrix.getValue(activities[i], activities[contextSize - 1]) / (contextSize - 1 - i);
					if (value > maxValue) {
						maxValue = value;
					}
				}
			}
			sumValue += maxValue;
		}
		return sumValue / (trace.size() - 1);
	}

	public String getName() {
		return "Context Average Causality";
	}

}
