package org.processmining.causalactivitymatrix.filters.impl;

import org.deckfour.xes.classification.XEventClass;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XTrace;
import org.processmining.causalactivitymatrix.filters.CausalityBasedFilterParameters;
import org.processmining.causalactivitymatrix.models.CausalActivityMatrix;

public class CausalityBasedSingleChangeFilter extends CausalityBasedAbstractFilter {

	public double getValue(XTrace trace, CausalActivityMatrix matrix, CausalityBasedFilterParameters parameters) {
		// TODO Auto-generated method stub
		XEventClass[] activities = new XEventClass[3];
		activities[1] = null;
		activities[2] = null;
		int count = 0;
		for (XEvent event : trace) {
			activities[0] = activities[1];
			activities[1] = activities[2];
			activities[2] = new XEventClass(parameters.getClassifier().getClassIdentity(event), 0);
			if (activities[1] != null) {
				double value = matrix.getValue(activities[1], activities[2]);
				double extraValue = -1.0;
				if (activities[0] != null) {
					extraValue = 2 * matrix.getValue(activities[0], activities[2])
							- matrix.getValue(activities[0], activities[1]);
				}
				/*
				 * If extraValue > value, then dropping activities[1] from the
				 * trace would improve the result. As a result, we assume that
				 * activities[1[] should be removed: noise.
				 */
				if (extraValue <= value) {
					for (XEventClass extraActivity : matrix.getActivities()) {
						extraValue = (matrix.getValue(activities[1], extraActivity) + matrix.getValue(extraActivity,
								activities[2])) / 2;
						/*
						 * If extraValue > value, then adding extraActivity in
						 * between activity[1] and activity[2] would improve the
						 * result. Noise.
						 */
						if (extraValue > value) {
							continue;
						}
					}
				}
				if (extraValue > value) {
					count++;
				}
			}
		}
		return (-1.0 * count) / trace.size();
	}

	public String getName() {
		return "Causality Single CHange";
	}

}
