package org.processmining.causalactivitymatrix.modifiers.impl;

import java.util.HashSet;

import org.deckfour.xes.classification.XEventClass;
import org.processmining.causalactivitymatrix.models.CausalActivityMatrix;
import org.processmining.causalactivitymatrix.models.impl.CausalActivityMatrixFactory;
import org.processmining.causalactivitymatrix.modifiers.CausalActivityMatrixModifier;

public class CausalActivityMatrixConcurrencyRatioModifier implements CausalActivityMatrixModifier {

	private double value;
	
	public CausalActivityMatrixConcurrencyRatioModifier(double value) {
		this.value = value;
	}
	
	public CausalActivityMatrix apply(CausalActivityMatrix matrix) {
		return value != 0.0 ? apply(matrix, value) : matrix;
	}

	private CausalActivityMatrix apply(CausalActivityMatrix matrix, double concurrencyRatio) {
		CausalActivityMatrix newMatrix = CausalActivityMatrixFactory.createCausalActivityMatrix();
		newMatrix.init(matrix.getLabel(), new HashSet<XEventClass>(matrix.getActivities()));
		for (XEventClass activity : matrix.getActivities()) {
			for (XEventClass otherActivity : matrix.getActivities()) {
				double value = matrix.getValue(activity, otherActivity);
				double otherValue = matrix.getValue(otherActivity, activity);
				if (value > 0.0 && otherValue > 0.0 && !activity.equals(otherActivity)
						&& Math.abs(value - otherValue) < concurrencyRatio) {
					newMatrix.setValue(activity, otherActivity, -0.5);
				} else {
					newMatrix.setValue(activity, otherActivity, value);
				}
			}
		}
		return newMatrix;
	}

}
