package org.processmining.causalactivitymatrix.modifiers.impl;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;

import org.deckfour.xes.classification.XEventClass;
import org.processmining.causalactivitymatrix.models.CausalActivityMatrix;
import org.processmining.causalactivitymatrix.models.impl.CausalActivityMatrixFactory;
import org.processmining.causalactivitymatrix.modifiers.CausalActivityMatrixModifier;

public class CausalActivityMatrixIncludeThresholdModifier implements CausalActivityMatrixModifier {

	private double value;
	
	public CausalActivityMatrixIncludeThresholdModifier(double value) {
		this.value = value;
	}
	
	public CausalActivityMatrix apply(CausalActivityMatrix matrix) {
		return value != 0.0 ? apply(matrix, value) : matrix;
	}

	private CausalActivityMatrix apply(CausalActivityMatrix matrix, double includeThreshold) {
		CausalActivityMatrix newMatrix = CausalActivityMatrixFactory.createCausalActivityMatrix();
		newMatrix.init(matrix.getLabel(), new HashSet<XEventClass>(matrix.getActivities()));
		Map<XEventClass, Double> maxValues = new HashMap<XEventClass, Double>();
		for (XEventClass activity : matrix.getActivities()) {
			maxValues.put(activity, matrix.getValue(activity, activity));
		}
		for (XEventClass activity : matrix.getActivities()) {
			for (XEventClass otherActivity : matrix.getActivities()) {
				double value = matrix.getValue(activity, otherActivity);
				if (value > maxValues.get(activity)) {
					maxValues.put(activity, value);
				}
				if (value > maxValues.get(otherActivity)) {
					maxValues.put(otherActivity, value);
				}
			}
		}
		for (XEventClass activity : matrix.getActivities()) {
			for (XEventClass otherActivity : matrix.getActivities()) {
				double maxValue = Math.min(maxValues.get(activity), maxValues.get(otherActivity));
				double value = matrix.getValue(activity, otherActivity);
				if (maxValue < 0.0) {
					value += includeThreshold;
					if (value == maxValue) {
						value = 0.0;
					} else if (value > maxValue) {
						value = (value >= 1.0 ? 1.0 : (value - maxValue) / (1.0 - maxValue));
					} else {
						value = (value <= -1.0 ? -1.0 : (value - maxValue) / (1.0 + maxValue));
					}
				}
				newMatrix.setValue(activity, otherActivity, value);
			}
		}
		return newMatrix;
	}
}
