package org.processmining.causalactivitymatrix.modifiers.impl;

import java.util.HashSet;

import org.deckfour.xes.classification.XEventClass;
import org.processmining.causalactivitymatrix.models.CausalActivityMatrix;
import org.processmining.causalactivitymatrix.models.impl.CausalActivityMatrixFactory;
import org.processmining.causalactivitymatrix.modifiers.CausalActivityMatrixModifier;

public class CausalActivityMatrixZeroValueModifier implements CausalActivityMatrixModifier {

	private double value;
	
	public CausalActivityMatrixZeroValueModifier(double value) {
		this.value = value;
	}
	
	public CausalActivityMatrix apply(CausalActivityMatrix matrix) {
		return value != 0.0 ? apply(matrix, value) : matrix;
	}

	public CausalActivityMatrix apply(CausalActivityMatrix matrix, double zeroValue) {
		CausalActivityMatrix newMatrix = CausalActivityMatrixFactory.createCausalActivityMatrix();
		newMatrix.init(matrix.getLabel(), new HashSet<XEventClass>(matrix.getActivities()));
		for (XEventClass activity : matrix.getActivities()) {
			for (XEventClass otherActivity : matrix.getActivities()) {
				double value = matrix.getValue(activity, otherActivity);
				if (value == zeroValue) {
					value = 0.0;
				} else if (value > zeroValue) {
					value = (value >= 1.0 ? 1.0 : (value - zeroValue) / (1.0 - zeroValue));
				} else {
					value = (value <= -1.0 ? -1.0 : (value - zeroValue) / (1.0 + zeroValue));
				}
				newMatrix.setValue(activity, otherActivity, value);
			}
		}
		return newMatrix;
	}
}
