package org.processmining.causalactivitymatrix.plugins;

import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.processmining.causalactivitymatrix.algorithms.FilterLogArrayUsingMatrixAlgorithm;
import org.processmining.causalactivitymatrix.dialogs.FilterLogUsingMatrixDialog;
import org.processmining.causalactivitymatrix.help.FilterLogArrayUsingMatrixHelp;
import org.processmining.causalactivitymatrix.models.CausalActivityMatrix;
import org.processmining.causalactivitymatrix.parameters.FilterLogUsingMatrixParameters;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.log.models.EventLogArray;

@Plugin(name = "Filter Using Matrix (Multiple Logs)", parameterLabels = { "Logs", "Causal Activity Matrix", "Parameters" }, returnLabels = { "Filtered Logs" }, returnTypes = { EventLogArray.class }, help = FilterLogArrayUsingMatrixHelp.TEXT)
public class FilterLogArrayUsingMatrixPlugin extends FilterLogArrayUsingMatrixAlgorithm {

	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "H.M.W. Verbeek", email = "h.m.w.verbeek@tue.nl")
	@PluginVariant(variantLabel = "Filter Using Matrix (Multiple Logs), UI", requiredParameterLabels = { 0, 1 })
	public EventLogArray runUI(UIPluginContext context, EventLogArray logs, CausalActivityMatrix matrix) {
		if (logs.getSize() > 0) {
			FilterLogUsingMatrixParameters parameters = new FilterLogUsingMatrixParameters(logs.getLog(0));
			FilterLogUsingMatrixDialog dialog = new FilterLogUsingMatrixDialog(logs.getLog(0), parameters);
			InteractionResult result = context.showWizard("Configure filter (filter, threshold)", true, true, dialog);
			if (result != InteractionResult.FINISHED) {
				return null;
			}
			return apply(context, logs, matrix, parameters);
		}
		return null;
	}

	/**
	 *  @deprecated Use runUI() instead.
	 */
	@Deprecated
	public EventLogArray filterUI(UIPluginContext context, EventLogArray logs, CausalActivityMatrix matrix) {
		return runUI(context, logs, matrix);
	}
	
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "H.M.W. Verbeek", email = "h.m.w.verbeek@tue.nl")
	@PluginVariant(variantLabel = "Filter Using Matrix (Multiple Logs), Default", requiredParameterLabels = { 0, 1 })
	public EventLogArray runDefault(PluginContext context, EventLogArray logs, CausalActivityMatrix matrix) {
		if (logs.getSize() > 0) {
			FilterLogUsingMatrixParameters parameters = new FilterLogUsingMatrixParameters(logs.getLog(0));
			return apply(context, logs, matrix, parameters);
		}
		return null;
	}

	/**
	 *  @deprecated Use runDefault() instead.
	 */
	@Deprecated
	public EventLogArray filterDefault(PluginContext context, EventLogArray logs, CausalActivityMatrix matrix) {
		return runDefault(context, logs, matrix);
	}
	
	@PluginVariant(variantLabel = "Filter Using Matrix (Multiple Logs), Parameters", requiredParameterLabels = { 0, 1, 2 })
	public EventLogArray run(PluginContext context, EventLogArray logs, CausalActivityMatrix matrix,
			FilterLogUsingMatrixParameters parameters) {
		if (logs.getSize() > 0) {
			return apply(context, logs, matrix, parameters);
		}
		return null;
	}

	/**
	 *  @deprecated Use run() instead.
	 */
	@Deprecated
	public EventLogArray filterParameters(PluginContext context, EventLogArray logs, CausalActivityMatrix matrix,
			FilterLogUsingMatrixParameters parameters) {
		return run(context, logs, matrix, parameters);
	}
	
}
