package org.processmining.causalactivitymatrix.plugins;

import java.util.Collection;

import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.deckfour.xes.model.XLog;
import org.processmining.causalactivitymatrix.algorithms.FilterLogUsingMatrixAlgorithm;
import org.processmining.causalactivitymatrix.connections.FilterLogUsingMatrixConnection;
import org.processmining.causalactivitymatrix.dialogs.FilterLogUsingMatrixDialog;
import org.processmining.causalactivitymatrix.help.FilterLogUsingMatrixHelp;
import org.processmining.causalactivitymatrix.models.CausalActivityMatrix;
import org.processmining.causalactivitymatrix.parameters.FilterLogUsingMatrixParameters;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;

@Plugin(name = "Filter Using Matrix (Single Log)", parameterLabels = { "Log", "Causal Activity Matrix", "Parameters" }, returnLabels = { "Filtered Log" }, returnTypes = { XLog.class }, help = FilterLogUsingMatrixHelp.TEXT)
public class FilterLogUsingMatrixPlugin extends FilterLogUsingMatrixAlgorithm {

	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "H.M.W. Verbeek", email = "h.m.w.verbeek@tue.nl")
	@PluginVariant(variantLabel = "Filter Using Matrix (Single Log), UI", requiredParameterLabels = { 0, 1 })
	public XLog runUI(UIPluginContext context, XLog log, CausalActivityMatrix matrix) {
		FilterLogUsingMatrixParameters parameters = new FilterLogUsingMatrixParameters(log);
		FilterLogUsingMatrixDialog dialog = new FilterLogUsingMatrixDialog(log, parameters);
		InteractionResult result = context.showWizard("Configure filter (filter, threshold)", true, true, dialog);
		if (result != InteractionResult.FINISHED) {
			return null;
		}
		return runConnections(context, log, matrix, parameters);
	}

	/**
	 *  @deprecated Use runUI() instead.
	 */
	@Deprecated
	public XLog filterUI(UIPluginContext context, XLog log, CausalActivityMatrix matrix) {
		return runUI(context, log, matrix);
	}
	
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "H.M.W. Verbeek", email = "h.m.w.verbeek@tue.nl")
	@PluginVariant(variantLabel = "Filter Using Matrix (Single Log), Default", requiredParameterLabels = { 0, 1 })
	public XLog runDefault(PluginContext context,XLog log, CausalActivityMatrix matrix) {
		FilterLogUsingMatrixParameters parameters = new FilterLogUsingMatrixParameters(log);
		return runConnections(context, log, matrix, parameters);
	}

	/**
	 *  @deprecated Use runDefault() instead.
	 */
	@Deprecated
	public XLog filterDefault(PluginContext context,XLog log, CausalActivityMatrix matrix) {
		return runDefault(context, log, matrix);
	}
	
	@PluginVariant(variantLabel = "Filter Using Matrix (Single Log), Parameters", requiredParameterLabels = { 0, 1, 2 })
	public XLog run(PluginContext context, XLog log, CausalActivityMatrix matrix,
			FilterLogUsingMatrixParameters parameters) {
		return runConnections(context, log, matrix, parameters);
	}

	/**
	 *  @deprecated Use run() instead.
	 */
	@Deprecated
	public XLog filterParameters(PluginContext context, XLog log, CausalActivityMatrix matrix,
			FilterLogUsingMatrixParameters parameters) {
		return run(context, log, matrix, parameters);
	}
	
	private XLog runConnections(PluginContext context, XLog log, CausalActivityMatrix matrix,
			FilterLogUsingMatrixParameters parameters) {
		if (parameters.isTryConnections()) {
			Collection<FilterLogUsingMatrixConnection> connections;
			try {
				connections = context.getConnectionManager().getConnections(
						FilterLogUsingMatrixConnection.class, context, log, matrix);
				for (FilterLogUsingMatrixConnection connection : connections) {
					if (connection.getObjectWithRole(FilterLogUsingMatrixConnection.LOG)
							.equals(log) && connection.getObjectWithRole(FilterLogUsingMatrixConnection.MATRIX)
							.equals(matrix) && connection.getParameters().equals(parameters)) {
						return connection
								.getObjectWithRole(FilterLogUsingMatrixConnection.FILTEREDLOG);
					}
				}
			} catch (ConnectionCannotBeObtained e) {
			}
		}
		XLog filteredLog = apply(context, log, matrix, parameters);
		if (parameters.isTryConnections()) {
			context.getConnectionManager().addConnection(
					new FilterLogUsingMatrixConnection(log, matrix, filteredLog, parameters));
		}
		return filteredLog;
	}
}
