package org.processmining.causalactivitymatrix.plugins;

import java.util.Collection;

import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.processmining.causalactivitymatrix.algorithms.ModifyCausalActivityMatrixAlgorithm;
import org.processmining.causalactivitymatrix.connections.ModifyCausalActivityMatrixConnection;
import org.processmining.causalactivitymatrix.dialogs.ModifyCausalActivityMatrixDialog;
import org.processmining.causalactivitymatrix.help.ModifyCausalActivityMatrixHelp;
import org.processmining.causalactivitymatrix.models.CausalActivityMatrix;
import org.processmining.causalactivitymatrix.parameters.ModifyCausalActivityMatrixParameters;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;

@Plugin(name = "Modify Matrix", parameterLabels = { "Causal Activity Matrix", "Parameters" }, returnLabels = { "Causal Activity Matrix" }, returnTypes = { CausalActivityMatrix.class }, help = ModifyCausalActivityMatrixHelp.TEXT)
public class ModifyCausalActivityMatrixPlugin extends ModifyCausalActivityMatrixAlgorithm {

	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "H.M.W. Verbeek", email = "h.m.w.verbeek@tue.nl")
	@PluginVariant(variantLabel = "Modify Matrix, UI", requiredParameterLabels = { 0 })
	public CausalActivityMatrix runUI(UIPluginContext context, CausalActivityMatrix matrix) {
		//		ConvertCausalActivityMatrixToCausalActivityGraphParameters parameters = new ConvertCausalActivityMatrixToCausalActivityGraphParameters();
		ModifyCausalActivityMatrixParameters parameters = new ModifyCausalActivityMatrixParameters();
		ModifyCausalActivityMatrixDialog dialog = new ModifyCausalActivityMatrixDialog(
				context, matrix, parameters);
		InteractionResult result = context.showWizard("Configure filter (thresholds)", true, true, dialog);
		if (result != InteractionResult.FINISHED) {
			return null;
		}
		return runConnections(context, matrix, parameters);
	}

	/**
	 * @deprecated Use runUI() instead.
	 */
	@Deprecated
	public CausalActivityMatrix convertUI(UIPluginContext context, CausalActivityMatrix matrix) {
		return runUI(context, matrix);
	}
	
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "H.M.W. Verbeek", email = "h.m.w.verbeek@tue.nl")
	@PluginVariant(variantLabel = "Modify Matrix, Default", requiredParameterLabels = { 0 })
	public CausalActivityMatrix runDefault(PluginContext context, CausalActivityMatrix matrix) {
		ModifyCausalActivityMatrixParameters parameters = new ModifyCausalActivityMatrixParameters();
		return runConnections(context, matrix, parameters);
	}

	/**
	 * @deprecated Use runDefault() instead.
	 */
	@Deprecated
	public CausalActivityMatrix convertDefault(PluginContext context, CausalActivityMatrix matrix) {
		return runDefault(context, matrix);
	}
	
	@PluginVariant(variantLabel = "Modify Matrix, Parameters", requiredParameterLabels = { 0, 1 })
	public CausalActivityMatrix run(PluginContext context, CausalActivityMatrix matrix,
			ModifyCausalActivityMatrixParameters parameters) {
		return runConnections(context, matrix, parameters);
	}

	/**
	 * @deprecated Use run() instead.
	 */
	@Deprecated
	public CausalActivityMatrix convertParameters(PluginContext context, CausalActivityMatrix matrix,
			ModifyCausalActivityMatrixParameters parameters) {
		return run(context, matrix, parameters);
	}

	private CausalActivityMatrix runConnections(PluginContext context, CausalActivityMatrix matrix,
			ModifyCausalActivityMatrixParameters parameters) {
		if (parameters.isTryConnections()) {
			Collection<ModifyCausalActivityMatrixConnection> connections;
			try {
				connections = context.getConnectionManager().getConnections(
						ModifyCausalActivityMatrixConnection.class, context, matrix);
				for (ModifyCausalActivityMatrixConnection connection : connections) {
					if (connection.getObjectWithRole(ModifyCausalActivityMatrixConnection.MATRIX)
							.equals(matrix) && connection.getParameters().equals(parameters)) {
						return connection
								.getObjectWithRole(ModifyCausalActivityMatrixConnection.MODIFIEDMATRIX);
					}
				}
			} catch (ConnectionCannotBeObtained e) {
			}
		}
		CausalActivityMatrix modifiedMatrix = apply(context, matrix, parameters);
		if (parameters.isTryConnections()) {
			context.getConnectionManager().addConnection(
					new ModifyCausalActivityMatrixConnection(matrix, modifiedMatrix, parameters));
		}
		return modifiedMatrix;
	}

}
